@@ -1,7 +1,7 @@
 #!/bin/bash - 
 
 # Copyright(c) 2011 LinBox
-# Written by BB <bboyer@imag.fr>
+# Written by Brice Boyer (briceboyer) <boyer.brice@gmail.com>
 # see COPYING for more details.
 
 
@@ -13,12 +13,20 @@
 # TODO : rpath instead of LD_LIBRARY_PATH ?
 
 STABLE_FFLAS=1.6.0
-STABLE_GIVARO=3.7.0
-GIV_TAR=207
-GIV_MD5=208
+STABLE_GIVARO=3.7.2
+GIV_TAR=370
+GIV_MD5=371
+
+
+function decompress {
+#tar xf $1
+gunzip -c $1 | tar xf -
+}
+
+
 
 #switches
-STABLE_VAR="true"
+STABLE_VAR="false"
 DEBUG=""
 DEBUG_VAR=""
 WARNINGS=""
@@ -27,7 +35,7 @@ OPTIM="--enable-optimization"
 OPTIM_VAR=""
 CHECK_VAR=""
 #options
-PREFIX_LOC="/tmp"
+PREFIX_LOC="/usr/local"
 PREFIX_VAR=""
 PREFIX="--prefix=$PREFIX_LOC"
 BLAS=""
@@ -77,7 +85,7 @@ help() {
 	echo
 	echo " * usage :"
 	echo
-	echo " --stable=[yes,no]     : install latest stable versions or latest svn versions."
+	echo " --stable=[yes,no]     : install latest stable versions or latest git versions."
 	echo "                         Default : yes, even if switch ommitted. No argument means yes"
 
 	echo " --prefix=MY/PATH      : install all libraries under MY/PATH."
@@ -355,7 +363,7 @@ fi
 cool
 
 ####################
-#  fectch sources  #
+#  fetch sources  #
 ####################
 
 cd build ;
@@ -384,7 +392,7 @@ if [ "$STABLE_VAR" = "true" ]; then
 	fi
 else
 	OK=0 ;
-	svn co svn://scm.forge.imag.fr/var/lib/gforge/chroot/scmrepos/svn/givaro/trunk 2>&1 >/dev/null && OK=1 
+	git clone https://github.com/linbox-team/givaro.git 2>&1 >/dev/null && OK=1 
 	[ "$OK" = "1" ] &&  cool  || die 
 fi
 
@@ -412,7 +420,7 @@ if [ "$STABLE_VAR" = "true" ]; then
 	fi
 else
 	OK=0 ;
-	svn co svn://linalg.org/fflas-ffpack 2>&1 >/dev/null && OK=1 
+	git clone https://github.com/linbox-team/fflas-ffpack.git 2>&1 >/dev/null && OK=1 
 	[ "$OK" = "1" ] &&  cool  || die
 fi
 
@@ -426,7 +434,7 @@ fi
 OK=0
 if [ "$STABLE_VAR" = "true" ]; then
 	echo -en "${BEG}extracting Givaro..."| tee -a ../auto-install.log
-	tar xzf givaro-$STABLE_GIVARO.tar.gz  && OK=1
+	decompress givaro-$STABLE_GIVARO.tar.gz  && OK=1
 	[ "$OK" = "1" ] &&  cool   || die 
 fi
 
@@ -435,7 +443,7 @@ fi
 OK=0
 if [ "$STABLE_VAR" = "true" ]; then
 	echo -en "${BEG}extracting Fflas-Ffpack..."| tee -a ../auto-install.log
-	tar xzf fflas-ffpack-$STABLE_FFLAS.tar.gz  && OK=1
+	decompress fflas-ffpack-$STABLE_FFLAS.tar.gz  && OK=1
 	[ "$OK" = "1" ] &&  cool   || die
 fi
 
@@ -448,7 +456,7 @@ fi
 if [ "$STABLE_VAR" = "true" ]; then
 	cd givaro-$STABLE_GIVARO || die
 else
-	cd trunk/ || die
+	cd givaro/ || die
 fi
 
 if [ -f Makefile ] ; then
