@@ -1,5 +1,5 @@
 /* linbox/tests/test-field.h
- * Copyright (C) 2001, 2002 Bradford Hovinen
+ * Extracted and evolved by bds from test-generic.h, written by Bradford Hovinen <hovinen@cis.udel.edu>
  *
  * ========LICENCE========
  * This file is part of the library LinBox.
@@ -15,46 +15,46 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
+ * License along with this library; if not, write to tthe Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  * ========LICENCE========
- *.
- *
- * Extracted by bds from test-generic.h, written by Bradford Hovinen <hovinen@cis.udel.edu>
  */
 
 /*! @file tests/test-field.h
  * @ingroup tests
- * @brief tests field operations
- * @test  tests field operations
+ * @brief tests ring and field operations
+ * @test  tests ring and field operations
  */
 
 /*
-// top level test that doesn't use subtests.
+// top level test that doesn't use field_subtests.
+bool testRing
 bool testField
 
 // top level test that uses subtest testRandomIteratorStep.
 bool testRandomIterator
 
-// top level runBasicRingTests calls these subtests.
+// top level runBasicRingTests calls these field_subtests.
 bool testFieldNegation
 bool testFieldDistributivity
 bool testFieldAssociativity
 bool testFieldCharacteristic
 bool testGeometricSummation
-bool testArithmeticConsistency
+bool testRingArithmeticConsistency
 bool testAxpyConsistency
 bool testRanditerBasic
 
-// top level runFieldTests calls these subtests after runBasicRingTests.
+// top level runPIRTests calls these field_subtests after runBasicRingTests.
+bool testFieldCommutativity
+...
+
+// top level runFieldTests calls these field_subtests after runBasicRingTests.
 bool testFieldInversion
 bool testFieldCommutativity
+bool testInvDivConsistency
 bool testFreshmansDream
 bool testRingTrivia
 
-//called in subtest testArithmeticConsistency
-bool testRingArithmeticConsistency
-bool testInvDivConsistency
 */
 
 #ifndef __LINBOX_test_field_H
@@ -68,30 +68,28 @@ bool testInvDivConsistency
 
 #include "linbox/util/commentator.h"
 #include "linbox/util/field-axpy.h"
-#include "linbox/randiter/nonzero.h"
+#include <givaro/givranditer.h>
 //#include "linbox/vector/stream.h"
 #include "linbox/integer.h"
+// #include "linbox/field/givaro.h"
 
 #include "test-common.h"
 
-/* Modular exponentiation */
+/* Givaro::Modular exponentiation */
 using namespace std;
 
-// LinBox::Commentator commentator(std::cout);
-
 using LinBox::commentator ;
 
-
 template <class Field>
 typename Field::Element& expt (const Field &F, typename Field::Element &res, const typename Field::Element &a, LinBox::integer &n)
 {
 	if (n == 0) {
-		F.init (res, 1);
+		F.init (res);
 	}
 	else if (n == 1) {
 		F.assign (res, a);
 	}
-	else if (n[0] & 1) {
+	else if (Givaro::isOdd(n)) {
 		n -= 1;
 		expt (F, res, a, n);
 		F.mulin (res, a);
@@ -100,7 +98,7 @@ typename Field::Element& expt (const Field &F, typename Field::Element &res, con
 		n /= 2;
 		expt (F, res, a, n);
 		typename Field::Element tmp;
-		F.init(tmp,0);
+		F.init(tmp);
 		res = F.mul (tmp, res, res);
 	}
 
@@ -126,34 +124,41 @@ bool reportError(string rep, bool& flag)
  * Return true on success and false on failure
  */
 
-template<class Field>
-bool testField (Field &F, const char *title, bool fieldp = true)
+template<class Ring>
+bool testRing (Ring &F, const char *title, bool fieldp = true, bool runInitConvertIdentity=true)
 {
-	commentator().start (title, "testField", 5);
-	// ostream &report = std::cout ;
+	commentator().start (title, "testRing", 5);
 	ostream &report = commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 
-	typename Field::Element zero, one, mOne, two, three;
-	F.init(zero, 0); F.init(one, 1); F.init(two, 2); F.init(three, 3);
-	F.init(mOne);
-	F.neg(mOne,one);
+	typedef typename Ring::Element Element;
+	LinBox::integer p, q;
+	F.characteristic(p);
+	
+	Element zero, one, mOne, two, mTwo, three, five, six, eight;
+	F.init(zero); F.assign(zero, F.zero);
+	F.init(one); F.assign(one, F.one);
+	F.init(mOne); F.assign(mOne, F.mOne);
 
-	typename Field::Element a, b, c, d, e, f;
-	F.init(a,0); F.init(b,0); F.init(c,0); F.init(d,0); F.init(e,0); F.init(f,0);
+	F.init(two); F.add(two, one, one);
+	F.init(mTwo); F.add(mTwo, mOne, F.mOne);
+	F.init(three); F.add(three, two, one);
+	F.init(five); F.add(five, three, one); F.addin(five, one);
+	F.init(six); F.add(six, five, one);
+	F.init(eight); F.add(eight, six, one); F.addin(eight, one);
 
+	Element a, b, c, d, e, f;
+	int64_t z = 0;
+	F.init(a,z); F.init(b,z); F.init(c,z); F.init(d,z); F.init(e,z); F.init(f,z);
 
-	report << "Field self description: " << F.write (report) << endl;
-	//	report << "field Element 2: " << F.write (report, two) << endl;
+	F.write(report << " (Ring self description: ") << ')' << endl;
+	report << "Ring characteristic: " << p << endl;
 
 	LinBox::integer n, m;
 	bool pass = true, part_pass = true;
 
 	commentator().start ("\t--Testing characteristic/cardinality match");
 
-	F.characteristic (n);
-	F.cardinality (m);
-
-	if (n > 0 && !isPower (m, n)) part_pass = reportError("Characteristic, cardinality mismatch", pass);
+	if (p > 0 && q > 0 && !isPower (q, p)) part_pass = reportError("Characteristic, cardinality mismatch", pass);
 
 	commentator().stop (MSG_STATUS (part_pass));
 	commentator().progress ();
@@ -164,82 +169,87 @@ bool testField (Field &F, const char *title, bool fieldp = true)
 	commentator().start ("\t--Testing correctness of 0 and 1");
 	part_pass = true;
 
-	if (!F.isZero (zero)) {
+	if ( not F.isZero (zero) or not F.isZero(F.zero) ) 
 		part_pass = reportError( "isZero (0) is false", pass);
-	}
-	if (F.isZero (one)) part_pass = reportError( "isZero (1) is true", pass);
-	if (F.isOne (zero)) part_pass = reportError( "isOne (0) is true", pass);
-	if (!F.isOne (one)) {
+	
+	if ( F.isZero (one) or F.isZero(F.one) ) 
+		part_pass = reportError( "isZero (1) is true", pass);
+	if ( F.isOne (zero) or F.isOne(F.zero) ) 
+		part_pass = reportError( "isOne (0) is true", pass);
+	if ( not F.isOne (one) or not F.isOne(F.one) ) 
 		part_pass = reportError( "isOne (1) is false", pass);
-	}
-
-	if (!F.isZero (F.zero)) {
-		part_pass = reportError( "isZero (0) is false", pass);
-	}
-	if (F.isZero (F.one)) part_pass = reportError( "isZero (1) is true", pass);
-	if (F.isOne (F.zero)) part_pass = reportError( "isOne (0) is true", pass);
-	if (!F.isOne (F.one)) {
-		part_pass = reportError( "isOne (1) is false", pass);
-	}
-
-	if ( !F.areEqual(F.mOne,mOne))
+	if ( !F.areEqual(F.mOne,mOne)) {
 		part_pass = reportError( "isMOne (-One) is false", pass);
+	}
 
-	typename Field::Element mOneFromCst;
-    F.init(mOneFromCst, -1);
+/* this is not required of init
+	if (p > 0) {
+		Element mOneFromCst;
+		F.init(mOneFromCst, (LinBox::integer)(p-1));
 
-    if ( !F.areEqual(F.mOne,mOneFromCst))
-		part_pass = reportError( "isMOne (-1) is false", pass);
+		if ( !F.areEqual(F.mOne,mOneFromCst)) {
+			part_pass = reportError( "isMOne (p-1) is false", pass);
+		}
+	}
+*/
 
 
 	commentator().stop (MSG_STATUS (part_pass));
 	commentator().progress ();
 
+    if (runInitConvertIdentity) {
+                
 	commentator().start ("\t--Testing init/convert");
 	part_pass = true;
 
-	if (F.cardinality (m) <= 0)
-		n = 49193295;   // Just using some odd value
+	// test of 0..card-1 bijection
+	typename Ring::RandIter r (F);
+        r.random(a);
+	F.convert(n, a);
+	F.write(report, a) << " --(convert)--> " << n << endl;
+	F.init(b, n);
+	F.write(report << n << " --(init)--> ", b) << endl;
+	if (not F.areEqual(a, b)) part_pass = reportError( "F.init (b, F.convert(n, a)) != a", pass);
+
+#if 0
+	// test of prime subring bijection
+	if (p <= 0)
+		n = 0;
 	else
-		n = n/2-1;
-	if (!n) ++n;
-
-
+		n = rand()%p;
 	report << "Initial integer: " << n << endl;
-	F.init (a, n);  F.write ( report << "Result of init: ", a) << endl;
 
-	F.convert (m, a);
-	report << "Result of convert: " << m << endl;
+	F.init (a, (int64_t)n);  F.write ( report << "Result of init: ", a) << endl;
+	F.convert (m, a); report << "Result of convert: " << m << endl;
 
 	if (m != n) part_pass = reportError( "F.convert (m, F.init (a, n)) != n", pass);
+#endif
 
 	commentator().stop (MSG_STATUS (part_pass));
 	commentator().progress ();
-
-	commentator().start ("\t--Testing field arithmetic");
+        }
+        
+	commentator().start ("\t--Testing ring arithmetic");
 	part_pass = true;
 
-	F.init (b, n-2);
-	F.init (d, n-2);
-	F.init (e, 3);
-
 	F.add (a, three, two); F.write (report << "Result of 2 + 3: ", a) << endl;
 	F.assign (d, three);
 	F.addin (d, two); F.write (report << "Result of 2 + 3 (inplace): ", d) << endl;
 
-	if (!F.areEqual (a, F.init (f, 5)) || !F.areEqual (d, a))
+	if (!F.areEqual (a, F.assign (f, five)) || !F.areEqual (d, a))
 		part_pass = reportError( "Results of add are incorrect", pass);
 
 	F.neg (a, two); F.write (report << "Result of -2: ", a) << endl;
 	F.assign (d, two);
 	F.negin (d); F.write (report << "Result of -2 (inplace): ", d) << endl;
-	F.init (f, -2); F.write( report << "-2 via init: ", f) << endl;
+//F.write( report << "-2 mTwo: ", mTwo) << endl;
+	F.assign (f, mTwo); F.write( report << "-2 via init: ", f) << endl;
 
 	if (!F.areEqual (a, f) || !F.areEqual (d, a))
 		part_pass = reportError( "Results of neg are incorrect", pass);
 
 	F.sub (a, three, two); F.write (report << "Result of 3 - 2: ", a) << endl;
-	F.init (d, 3);
+	F.assign (d, three);
 	F.subin (d, two); F.write (report << "Result of 3 - 2 (inplace): ", d) << endl;
 
 	if (!F.areEqual (a, one) || !F.areEqual (d, a))
@@ -248,30 +258,15 @@ bool testField (Field &F, const char *title, bool fieldp = true)
 	F.mul (a, two, three); F.write (report << "Result of 2 * 3: ", a) << endl;
 	F.assign (d, two);
 	F.mulin (d, three); F.write (report << "Result of 2 * 3 (inplace): ", d) << endl;
-	F.init (f, 6);
+	F.assign (f, six);
 
 	if (!F.areEqual (a, f) || !F.areEqual (d, a))
 		part_pass = reportError( "Results of mul incorrect", pass);
 
-	F.inv (a, one); F.write (report << "Result of inverting 1: ", a) << endl;
-	F.assign (d, one);
-	F.invin (d); F.write (report << "Result of inverting 1 (inplace): ", d) << endl;
-
-	if (!F.areEqual (a, one) || !F.areEqual (d, a))
-		part_pass = reportError( "Results of inv incorrect", pass);
-
-	if ( F.isZero(two) ) F.assign(f, three); else F.assign(f, two);
-	F.div (a, f, f); F.write ( report << "Result of f/f: ", a) << endl;
-	F.assign(d, f);
-	F.divin (d, f); F.write ( report << "Result of f/f (inplace): ", d) << endl;
-
-	if (!F.areEqual (a, one) || !F.areEqual (d, a))
-		part_pass = reportError( "Results of div incorrect", pass);
-
 	F.axpy (a, two, three, two); F.write ( report << "Result of axpy 2*3 + 2: ", a) << endl;
 	F.assign (d, two);
 	F.axpyin (d, two, three); F.write ( report << "Result of axpy 2*3 + 2 (inplace): ", d) << endl;
-	F.init (f, 8);
+	F.assign (f, eight);
 
 	if ( !F.areEqual (a, f) || !F.areEqual (d, a) )
 		part_pass = reportError( "Results of axpy incorrect", pass);
@@ -284,26 +279,27 @@ bool testField (Field &F, const char *title, bool fieldp = true)
 	//,..
 	// 2^101 - 1 vs 1 + 2 + 4 + ... + 2^100
 
-	F.init (a, 1);
-	F.init (b, 1);
-	F.init (c, 0);
-
 	n = 101;
 	expt(F, a, two, n);
-	F.subin (a, one); F.write( report << "using expt, 2^101 - 1: ", a) << endl;
+	F.subin (a, one);
+	F.write( report << "using expt, 2^101 - 1: ", a) << endl;
 
-	for (int i = 1; i <= 101; ++i) {
-		F.addin (c, b);
-		F.mulin (b, two);
+	for (int i = 0; i < 101; ++i) {
+		F.mulin(c, two);
+		F.addin (c, one);
 	}
 
+	F.write( report << "using sum, 1 + 2 + .. + 2^100: ", c) << endl;
+
 	if (!F.areEqual (a, c))
 		part_pass = reportError( "2^101 - 1 != 1 + 2 + .. + 2^100", pass);
 
 	// 1 + 2*(1 + 2*( ... (1) ... )), 100 times.
 	F.assign (d, one);
-	for (int i = 1; i < 101; ++i)
-	{	F.axpy (b, two, d, one); F.assign(d, b); }
+	for (int i = 1; i < 101; ++i) {
+		F.axpy (b, two, d, one);
+		F.assign(d, b);
+	}
 	F.write( report << "using axpy, 1 + 2(1 + ... + 2(1)...), with 100 '+'s: ", d) << endl;
 
 	if (!F.areEqual (a, d))
@@ -321,15 +317,59 @@ bool testField (Field &F, const char *title, bool fieldp = true)
 	 * .
 	 */
 
-	commentator().stop (MSG_STATUS (pass), (const char *) 0, "testField");
+	commentator().stop (MSG_STATUS (pass), (const char *) 0, "testRing");
 
 	return pass;
 }
 
+template<class Field>
+bool testField (Field &F, const char *title, bool fieldp = true, bool runInitConvertIdentity=true)
+{
+	commentator().start (title, "testField", 5);
+	ostream &report = commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 
-/** Tests of algebraic properties of fields */
+	LinBox::integer p, q;
+	F.characteristic(p);
+	F.cardinality (q);
 
-namespace subtests {
+    typename Field::Element two, three;
+	F.init(two); F.add(two, F.one, F.one);
+	F.init(three); F.add(three, two, F.one);
+
+    typename Field::Element a, b, c, d, e, f;
+	F.init(a);F.init(b);F.init(c);F.init(d);F.init(e);F.init(f);
+        
+	commentator().start ("\t--Testing field arithmetic");
+	bool part_pass = true;
+
+	F.inv (a, F.one); F.write (report << "Result of inverting 1: ", a) << endl;
+	F.assign (d, F.one);
+	F.invin (d); F.write (report << "Result of inverting 1 (inplace): ", d) << endl;
+
+	if (!F.areEqual (a, F.one) || !F.areEqual (d, a))
+		part_pass = reportError( "Results of inv incorrect", part_pass);
+
+	if ( F.isZero(two) ) F.assign(f, three); else F.assign(f, two);
+	F.div (a, f, f); F.write ( report << "Result of f/f: ", a) << endl;
+	F.assign(d, f);
+	F.divin (d, f); F.write ( report << "Result of f/f (inplace): ", d) << endl;
+
+	if (!F.areEqual (a, F.one) || !F.areEqual (d, a))
+		part_pass = reportError( "Results of div incorrect", part_pass);
+	
+	commentator().stop (MSG_STATUS (part_pass));
+	commentator().progress ();
+
+
+	commentator().stop (MSG_STATUS (part_pass), (const char *) 0, "testField");
+
+        return part_pass & testRing(F,title,fieldp,runInitConvertIdentity);
+        
+}
+
+/** Tests of algebraic properties of rings and fields */
+
+namespace field_subtests {
 	/* Generic test 6: Negation of elements
 	 *
 	 * Negates random elements and checks that they are true negatives
@@ -345,7 +385,8 @@ namespace subtests {
 		commentator().start (st, "testFieldNegation", iterations);
 
 		typename Field::Element a, neg_a, neg_a_a, zero;
-		F.init(a,0); F.init(neg_a,0); F.init(neg_a_a,0); F.init (zero, 0);
+		int64_t z = 0;
+		F.init(a,z); F.init(neg_a,z); F.init(neg_a_a,z); F.init (zero, z);
 		typename Field::RandIter r (F);
 
 		bool ret = true;
@@ -394,11 +435,10 @@ namespace subtests {
 		strcpy (st, str.str().c_str());
 		commentator().start (st, "testFieldInversion", iterations);
 
-		typename Field::Element a, ainv, aainv, one;
-		F.init (a,0);
-		F.init (ainv,0);
-		F.init (aainv,0);
-		F.init (one, 1);
+		typename Field::Element a, ainv, aainv;
+		F.init (a,(int64_t)0);
+		F.init (ainv,(int64_t)0);
+		F.init (aainv,(int64_t)0);
 		typename Field::RandIter r (F);
 
 		bool ret = true;
@@ -421,7 +461,7 @@ namespace subtests {
 
 			report << "a a^{-1} = ";  F.write (report, aainv) << endl;
 
-			if (!F.areEqual (aainv, one)) reportError("a a^-1 != 1", ret);
+			if (!F.areEqual (aainv, F.one)) reportError("a a^-1 != 1", ret);
 
 			commentator().stop ("done");
 			commentator().progress ();
@@ -449,10 +489,11 @@ namespace subtests {
 		commentator().start (st, "testFieldDistributivity", iterations);
 
 		typename Field::Element a, b, c, a_b, a_bc, ac, bc, ac_bc, ca_b, ca, cb, ca_cb;
-		F.init (a,0); F.init (b,0); F.init (c,0);
-		F.init (a_b,0); F.init (a_bc,0); F.init (ac,0); F.init (bc,0);
-		F.init (ac_bc,0);
-		F.init (ca_b,0); F.init (ca,0); F.init (cb,0); F.init (ca_cb,0);
+		int64_t z = 0;
+		F.init (a,z); F.init (b,z); F.init (c,z);
+		F.init (a_b,z); F.init (a_bc,z); F.init (ac,z); F.init (bc,z);
+		F.init (ac_bc,z);
+		F.init (ca_b,z); F.init (ca,z); F.init (cb,z); F.init (ca_cb,z);
 
 		typename Field::RandIter r (F);
 
@@ -523,9 +564,9 @@ namespace subtests {
 		commentator().start (st, "testFieldCommutativity", iterations);
 
 		typename Field::Element a, b, ab, ba, a_b, b_a;
-		F.init (a,0); F.init (b,0);
-		F.init (ab,0); F.init (ba,0);
-		F.init (a_b,0); F.init (b_a,0);
+		F.init (a,(int64_t)0); F.init (b,(int64_t)0);
+		F.init (ab,(int64_t)0); F.init (ba,(int64_t)0);
+		F.init (a_b,(int64_t)0); F.init (b_a,(int64_t)0);
 
 
 		typename Field::RandIter r (F);
@@ -596,8 +637,8 @@ namespace subtests {
 		commentator().start (st, "testFieldAssociativity", iterations);
 
 		typename Field::Element a, b, c, a_b, b_c, a_bc, ab_c;
-		F.init (a,0); F.init (b,0); F.init (c,0);
-		F.init (a_b,0); F.init (b_c,0); F.init (a_bc,0); F.init (ab_c,0);
+		F.init (a,(int64_t)0); F.init (b,(int64_t)0); F.init (c,(int64_t)0);
+		F.init (a_b,(int64_t)0); F.init (b_c,(int64_t)0); F.init (a_bc,(int64_t)0); F.init (ab_c,(int64_t)0);
 		typename Field::RandIter r (F);
 
 		bool ret = true;
@@ -666,33 +707,29 @@ namespace subtests {
 		strcpy (st, str.str().c_str());
 		commentator().start (st, "testGeometricSummation", iterations);
 
-		typename Field::Element a, a_n, k, zero, one;
+		typename Field::Element a, a_n, k;
 		typename Field::RandIter r (F);
-		typename LinBox::NonzeroRandIter<Field> z(F,r);
+//		typename Givaro::GeneralRingNonZeroRandIter<Field> z(r);
 
-		F.init (zero, 0);
-		F.init (one, 1);
-		F.init (a,0); F.init (a_n,0); F.init (k,0);
+		F.init (a,(int64_t)0); F.init (a_n,(int64_t)0); F.init (k,(int64_t)0);
 
 		bool ret = true;
 		LinBox::Integer card; F.cardinality(card);
-
 		for (unsigned int i = 0; i < iterations; i++) {
 			commentator().startIteration (i);
 
-			size_t no_bad_loop = card+10 ;
-			do z.random (a); while (F.areEqual (a, one) && --no_bad_loop);
-			if (!no_bad_loop) {
-				reportError(" *** ERROR *** could not find an element different form 1...",ret);
-				break;
-			}
+//			size_t no_bad_loop = card+10 ;
+//			do z.random (a); while (F.areEqual (a, F.one) && --no_bad_loop);
+//			if (!no_bad_loop) {
+//				reportError(" *** ERROR *** could not find an element different from 1...",ret);
+//				break;
 
 			ostream &report = commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 			report << "Random element a: ";
 			F.write (report, a) << endl;
 
-			F.assign (k, one);
-			F.assign (a_n, a);
+			F.assign (k, F.zero);
+			F.assign (a_n, F.one);
 
 			for (unsigned int j = 0; j < n; ++j) {
 				F.addin (k, a_n);
@@ -707,15 +744,18 @@ namespace subtests {
 			report << "sum(a^i, i = 0..n-1) = ";
 			F.write (report, k) << endl;
 
-			F.subin (a_n, one);
+			F.subin (a_n, F.one);
 			report << "(a^n - 1) = ";
 			F.write (report, a_n) << endl;
 
-			F.subin (a, one);
+			F.subin (a, F.one);
 			report << "(a - 1) = ";
 			F.write (report, a) << endl;
 
-			F.divin (a_n, a);
+			if (not F.isZero(a))
+				F.divin (a_n, a);
+			else
+				F.mulin(k, a);
 
 			report << "(a^n - 1) / (a - 1) = ";
 			F.write (report, a_n) << endl;
@@ -749,12 +789,11 @@ namespace subtests {
 		commentator().start (string(str.str()).c_str(), "testFieldCharacteristic", iterations);
 
 		LinBox::integer p, j;
-		typename Field::Element a, sigma, zero;
+		typename Field::Element a, sigma;
 		typename Field::RandIter r (F);
 
 		F.characteristic (p);
-		F.init (zero, 0);
-		F.init (a,0); F.init (sigma,0);
+		F.init (a,(int64_t)0); F.init (sigma,(int64_t)0);
 
 		bool ret = true;
 
@@ -770,7 +809,7 @@ namespace subtests {
 			Report << "Random element a: ";
 			F.write (Report, a) << endl;
 
-			F.assign (sigma, zero);
+			F.assign (sigma, F.zero);
 
 			// suggestion: make this run in time O(lg(p)), then take the conditional out of the run...Tests
 			for (j = 0; j < p; j += 1)
@@ -821,8 +860,8 @@ namespace subtests {
 		typename Field::RandIter r (F);
 		typename Field::Element a, b, a_b, a_b_p, a_p, b_p, a_p_b_p;
 
-		F.init (a,0); F.init (b,0); F.init (a_b,0);
-		F.init (a_b_p,0); F.init (a_p,0); F.init (b_p,0); F.init (a_p_b_p,0);
+		F.init (a,(int64_t)0); F.init (b,(int64_t)0); F.init (a_b,(int64_t)0);
+		F.init (a_b_p,(int64_t)0); F.init (a_p,(int64_t)0); F.init (b_p,(int64_t)0); F.init (a_p_b_p,(int64_t)0);
 
 		for (unsigned int i = 0; i < iterations; i++) {
 			commentator().startIteration (i);
@@ -896,7 +935,7 @@ namespace subtests {
 
 		typename Field::RandIter r (F);
 		typename Field::Element a, b, c1, c2;
-		F.init (a,0); F.init (b,0); F.init (c1,0); F.init (c2,0);
+		F.init (a,(int64_t)0); F.init (b,(int64_t)0); F.init (c1,(int64_t)0); F.init (c2,(int64_t)0);
 
 		for (unsigned int i = 0; i < iterations; i++) {
 			commentator().startIteration (i);
@@ -975,7 +1014,8 @@ namespace subtests {
 
 		typename Field::RandIter r (F);
 		typename Field::Element a, b, c1, c2;
-		F.init (a,0); F.init (b,0); F.init (c1,0); F.init (c2,0);
+		uint64_t zero = 0;
+		F.init (a,zero); F.init (b,zero); F.init (c1,zero); F.init (c2,zero);
 
 		for (unsigned int i = 0; i < iterations; i++) {
 			commentator().startIteration (i);
@@ -1023,17 +1063,18 @@ namespace subtests {
 		return ret;
 	}
 
-
+/*
 	template <class Field>
 	bool testArithmeticConsistency (const Field &F, const char *name, unsigned int iterations)
 	{
 		bool ret = true ;
 
-		ret &= subtests::testRingArithmeticConsistency(F, name, iterations) ;
-		ret &= subtests::testInvDivConsistency(F, name, iterations);
+		ret &= field_subtests::testRingArithmeticConsistency(F, name, iterations) ;
+		ret &= field_subtests::testInvDivConsistency(F, name, iterations);
 
 		return ret;
 	}
+	*/
 
 
 	template <class Field>
@@ -1054,42 +1095,41 @@ namespace subtests {
 		ostream &rapport = commentator().report (LinBox::Commentator::LEVEL_UNIMPORTANT, INTERNAL_DESCRIPTION);
 
 		//!@todo enable init with 1UL et -1L pour GMPRationalElement
-		typename Field::Element one, mOne, zero ;
-		LinBox::integer pun = 1 ;
-		LinBox::integer mun = -1 ;
-		LinBox::integer zer = 0 ;
-		F.init(one,pun);
-		F.init(mOne,mun);
-		F.neg(mOne,one);
-		// F.init(mOne,-1L);
-		F.init(zero,zer);
+		//typename Field::Element one, mOne, zero ;
+		//LinBox::integer pun = 1 ;
+		//LinBox::integer zer = 0 ;
+		//F.init(one,pun);
+		//F.neg(mOne,one);
+		//F.init(zero,zer);
 
 		rapport << "1 - 1 = " ;
 		typename Field::Element nil ;
+		F.init(nil);
 
-		F.add(nil,one,mOne);
+		F.add(nil,F.one,F.mOne);
 
 		F.write(rapport,nil) << std::endl ;
 
-		if ( !F.areEqual(nil,zero) ) {
-			reportError("1+1!=0", ret);
+		if ( !F.areEqual(nil,F.zero) ) {
+			reportError("1+-1!=0", ret);
 		}
 
 		typename Field::Element un ;
+		F.init(un);
 		rapport << "-1 * -1 = " ;
-		F.mul(un,mOne,mOne);
+		F.mul(un,F.mOne,F.mOne);
 		F.write(rapport,un) << std::endl ;
 
-		if ( !F.areEqual(un,one) ) {
+		if ( !F.areEqual(un,F.one) ) {
 			reportError("-1 * -1 != 1", ret);
 		}
 
-		F.init(nil,pun);
+		//F.init(nil,pun);
 		rapport << "-1 +  -1 * -1 = " ;
-		F.axpy(nil,mOne,mOne,mOne) ;
+		F.axpy(nil,F.mOne,F.mOne,F.mOne) ;
 		F.write(rapport,nil) << std::endl ;
 
-		if ( !F.areEqual(nil,zero) ) {
+		if ( !F.areEqual(nil,F.zero) ) {
 			reportError("-1+(-1*-1)!=0", ret);
 		}
 
@@ -1120,8 +1160,8 @@ namespace subtests {
 
 		typename Field::RandIter r (F);
 		typename Field::Element a, x, y, c1, c2, c3;
-		F.init (a,0); F.init (x,0); F.init (y,0);
-		F.init (c1,0); F.init (c2,0); F.init (c3,0);
+		F.init (a,(int64_t)0); F.init (x,(int64_t)0); F.init (y,(int64_t)0);
+		F.init (c1,(int64_t)0); F.init (c2,(int64_t)0); F.init (c3,(int64_t)0);
 
 		for (unsigned int i = 0; i < iterations; i++) {
 			commentator().startIteration (i);
@@ -1175,7 +1215,7 @@ namespace subtests {
 
 		typename Field::RandIter r (F);
 		typename Field::Element a;
-		F.init (a,0);
+		F.init (a,(int64_t)0);
 
 		if (iterations < 20) iterations = 20;
 		for (unsigned int i = 0; i < iterations; i++) {
@@ -1188,54 +1228,54 @@ namespace subtests {
 		delete[] st;
 		return ret;
 	}
-} // namespace subtests
+} // namespace field_subtests
 
-	/* Convenience function to run all of the field tests on a given field */
-	template <class Field>
-	bool runBasicRingTests (const Field &F, const char *desc, unsigned int iterations, bool runCharacteristicTest = true)
-	{
-		bool pass = true;
-		ostringstream str;
+/* Convenience function to run all of the basic ring tests */
+template <class Ring>
+bool runBasicRingTests (const Ring &F, const char *desc, unsigned int iterations, bool runCharacteristicTest = true, bool runInitConvertIdentity=true)
+{
+	bool pass = true;
+	ostringstream str;
 
-		str << "\t--Testing " << desc << " ring" << ends;
-		char * st = new char[str.str().size()];
-		strcpy (st, str.str().c_str());
+	str << "\t--Testing " << desc << " ring" << ends;
+	char * st = new char[str.str().size()];
+	strcpy (st, str.str().c_str());
 
-		commentator().start (st, "runBasicRingTests", runCharacteristicTest ? 11 : 10);
+	commentator().start (st, "runBasicRingTests", runCharacteristicTest ? 11 : 10);
 
-		if (!testField                     (F, string(str.str()).c_str()))           pass = false;
-		commentator().progress ();
-		if (!subtests::testFieldNegation             (F, desc, iterations))                    pass = false;
-		commentator().progress ();
-		if (!subtests::testFieldDistributivity       (F, desc, iterations))                    pass = false;
-		commentator().progress ();
-		if (!subtests::testFieldAssociativity        (F, desc, iterations))                    pass = false;
+	if (!testField                     (F, string(str.str()).c_str(),true,runInitConvertIdentity))           pass = false;
+	commentator().progress ();
+	if (!field_subtests::testFieldNegation             (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+	if (!field_subtests::testFieldDistributivity       (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+	if (!field_subtests::testFieldAssociativity        (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+
+	if (runCharacteristicTest) {
+		if (!field_subtests::testFieldCharacteristic (F, desc, iterations))                  pass = false;
 		commentator().progress ();
+	}
+	LinBox::integer card;
 
-		if (runCharacteristicTest) {
-			if (!subtests::testFieldCharacteristic (F, desc, iterations))                  pass = false;
-			commentator().progress ();
-		}
-		LinBox::integer card;
-
-		if (F.cardinality(card) != 2) { // otherwise it is not very interesting to find a element not zero and not one !
-			if (!subtests::testGeometricSummation        (F, desc, iterations, 100))               pass = false;
-			commentator().progress ();
-		}
-
-		if (!subtests::testArithmeticConsistency (F, desc, iterations))                    pass = false;
+	if (F.cardinality(card) != 2) { // otherwise it is not very interesting to find a element not zero and not one !
+		if (!field_subtests::testGeometricSummation        (F, desc, iterations, 100))               pass = false;
 		commentator().progress ();
-		if (!subtests::testAxpyConsistency           (F, desc, iterations))                    pass = false;
-		commentator().progress ();
-		if (!subtests::testRanditerBasic             (F, desc, iterations))                    pass = false;
-		commentator().progress ();
-
-		commentator().stop (MSG_STATUS (pass), (const char *) 0, "runBasicRingTests");
-		delete[] st;
-		return pass;
 	}
 
-namespace subtests {
+	if (!field_subtests::testRingArithmeticConsistency (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+	if (!field_subtests::testAxpyConsistency           (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+	if (!field_subtests::testRanditerBasic             (F, desc, iterations))                    pass = false;
+	commentator().progress ();
+
+	commentator().stop (MSG_STATUS (pass), (const char *) 0, "runBasicRingTests");
+	delete[] st;
+	return pass;
+}
+
+namespace field_subtests {
 	/* Random number test
 	 *
 	 * Test that the random iterator over the given field works
@@ -1261,6 +1301,7 @@ namespace subtests {
 
 		LinBox::integer card;
 		unsigned int i;
+//                std::cerr<<"num_categories = "<<num_categories<<std::endl;
 		std::vector<int> categories1 (num_categories, 0);
 		std::vector<int> categories2 (num_categories, 0);
 		std::list<std::vector<int> > diff_categories;
@@ -1282,17 +1323,21 @@ namespace subtests {
 		for (i = 0; i < num_trials; ++i) {
 			LinBox::integer ix, id;
 			F.convert(ix, iter.random (x));
-
-
-			categories1[ix % num_categories]++;
-			categories2[(unsigned int) (double (ix) / double (card) * num_categories)]++;
+                        
+                        LinBox::Integer ix2 = ix % num_categories;
+                        if (ix2<0) ix2+=num_categories;
+			categories1[ix2]++;
+			categories2[(unsigned int) (double (ix2) / double (card) * num_categories) %num_categories]++;
 
 			typename std::list<typename Field::Element>::iterator x_queue_iter = x_queue.begin ();
 			diff_cat_iter = diff_categories.begin ();
 
+//                        std::cerr<<x_queue.size()<<" "<<diff_categories.size()<<std::endl;
 			for (; x_queue_iter != x_queue.end (); ++x_queue_iter, ++diff_cat_iter) {
 				F.convert(id, F.sub (d, *x_queue_iter, x));
-				(*diff_cat_iter)[id % num_categories]++;
+                                id %= num_categories;
+                                if (id<0) id += num_categories;
+				(*diff_cat_iter)[(size_t) id]++;
 			}
 
 			x_queue.push_front (x);
@@ -1356,10 +1401,10 @@ namespace subtests {
 		//commentator().stop (MSG_STATUS (ret), (const char *) 0, "testRandomIteratorStep");
 		return ret;
 	}
-}
+}// namespace field_subtests
 
 template <class Field>
-bool runFieldTests (const Field &F, const char *desc, unsigned int iterations, size_t n, bool runCharacteristicTest = true)
+bool runFieldTests (const Field &F, const char *desc, unsigned int iterations, size_t n, bool runCharacteristicTest = true, bool runInitConvertIdentity=true)
 	// n is not used.
 {
 	ostringstream str;
@@ -1368,12 +1413,12 @@ bool runFieldTests (const Field &F, const char *desc, unsigned int iterations, s
 	char * st = new char[str.str().size()];
 	strcpy (st, str.str().c_str());
 	commentator().start (st, "runFieldTests");
-	bool ret =  runBasicRingTests(F, desc, iterations, runCharacteristicTest) ;
-	//ret &= subtests::testInvDivConsistency(F, desc, iterations) ; // it's called in runBasicRingTests
-	ret &= subtests::testFieldInversion (F, desc, iterations) ;
-	ret &= subtests::testFieldCommutativity (F, desc, iterations) ;
-	ret &= subtests::testFreshmansDream(F, desc, iterations);
-	ret &= subtests::testRingTrivia(F,desc);
+	bool ret =  runBasicRingTests(F, desc, iterations, runCharacteristicTest, runInitConvertIdentity) ;
+	ret &= field_subtests::testInvDivConsistency(F, desc, iterations) ;
+	ret &= field_subtests::testFieldInversion (F, desc, iterations) ;
+	ret &= field_subtests::testFieldCommutativity (F, desc, iterations) ;
+	ret &= field_subtests::testFreshmansDream(F, desc, iterations);
+	ret &= field_subtests::testRingTrivia(F,desc);
 
 	commentator().stop (MSG_STATUS (ret));
 	delete[] st;
@@ -1408,7 +1453,7 @@ bool testRandomIterator (const Field &F, const char *text,
 
 	/* This test either passes or runs a lot of times */
 	for (int i = 1;
-	     (!  subtests::testRandomIteratorStep (F, text, num_trials, num_categories, hist_len)) && (i <= 2) ;
+	     (!  field_subtests::testRandomIteratorStep (F, text, num_trials, num_categories, hist_len)) && (i <= 2) ;
 	     ++i ){
 		//if (0 == i % 5)
 			//std::ostream &report = commentator().report (LinBox::Commentator::LEVEL_NORMAL, INTERNAL_DESCRIPTION) << "Warning! Probable failure of uniformity" << std::endl;
@@ -1425,11 +1470,11 @@ bool testRandomIterator (const Field &F, const char *text,
 //@}
 #endif // __LINBOX_test_field_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
 
